package sun.security.util;

import java.io.IOException;
import java.security.spec.InvalidParameterSpecException;
import java.security.spec.ECGenParameterSpec;
import java.security.spec.AlgorithmParameterSpec;
import java.security.GeneralSecurityException;
import java.security.InvalidKeyException;
import java.security.AlgorithmParameters;
import java.security.spec.ECParameterSpec;
import java.security.AlgorithmParametersSpi;

public final class ECParameters extends AlgorithmParametersSpi
{
    private NamedCurve namedCurve;
    
    public static AlgorithmParameters getAlgorithmParameters(ECParameterSpec paramSpec) throws InvalidKeyException {
        try {
            AlgorithmParameters instance = AlgorithmParameters.getInstance("EC", "SunEC");
            instance.init(paramSpec);
            return instance;
        }
        catch (GeneralSecurityException cause) {
            throw new InvalidKeyException("EC parameters error", cause);
        }
    }
    
    public ECParameters() {
    }
    
    @Override
    protected void engineInit(AlgorithmParameterSpec obj) throws InvalidParameterSpecException {
        if (obj == null) {
            throw new InvalidParameterSpecException("paramSpec must not be null");
        }
        if ((obj instanceof NamedCurve)) {
            this.namedCurve = (NamedCurve)obj;
            return;
        }
        if ((obj instanceof ECParameterSpec)) {
            this.namedCurve = CurveDB.lookup((ECParameterSpec)obj);
        }
        else if ((obj instanceof ECGenParameterSpec)) {
            this.namedCurve = CurveDB.lookup(((ECGenParameterSpec)obj).getName());
        }
        else if ((obj instanceof ECKeySizeParameterSpec)) {
            this.namedCurve = CurveDB.lookup(((ECKeySizeParameterSpec)obj).getKeySize());
        }
        else {
            throw new InvalidParameterSpecException("Only ECParameterSpec and ECGenParameterSpec supported");
        }
        if (this.namedCurve == null) {
            throw new InvalidParameterSpecException("Not a supported curve: " + obj);
        }
        return;
    }
    
    @Override
    protected void engineInit(byte[] array) throws IOException {
        DerValue derValue = new DerValue(array);
        if (derValue.tag == 6) {
            ObjectIdentifier oid = derValue.getOID();
            NamedCurve lookup = CurveDB.lookup(oid.toString());
            if (lookup == null) {
                throw new IOException("Unknown named curve: " + oid);
            }
            this.namedCurve = lookup;
            return;
        }
        else {
            throw new IOException("Only named ECParameters supported");
        }
    }
    
    @Override
    protected void engineInit(byte[] array, String s) throws IOException {
        this.engineInit(array);
    }
    
    @Override
    protected <T extends AlgorithmParameterSpec> T engineGetParameterSpec(Class<T> clazz) throws InvalidParameterSpecException {
        if (clazz.isAssignableFrom(ECParameterSpec.class)) {
            return (T)clazz.cast(this.namedCurve);
        }
        if (clazz.isAssignableFrom(ECGenParameterSpec.class)) {
            return (T)clazz.cast(new ECGenParameterSpec(this.namedCurve.getObjectId()));
        }
        if (clazz.isAssignableFrom(ECKeySizeParameterSpec.class)) {
            return (T)clazz.cast(new ECKeySizeParameterSpec(this.namedCurve.getCurve().getField().getFieldSize()));
        }
        throw new InvalidParameterSpecException("Only ECParameterSpec and ECGenParameterSpec supported");
    }
    
    @Override
    protected byte[] engineGetEncoded() throws IOException {
        return this.namedCurve.getEncoded();
    }
    
    @Override
    protected byte[] engineGetEncoded(String s) throws IOException {
        return this.engineGetEncoded();
    }
    
    @Override
    protected String engineToString() {
        if (this.namedCurve == null) {
            return "Not initialized";
        }
        return this.namedCurve.toString();
    }
}
